# File: .configure.sh
# Shell: SH
# Ver: 1.0
# Purpose: To create .config, a file containing the user's preferences
# *Not* in tcsh, sorry.  I have more familiarity with sh.

echo ""
echo "Configuration..."

echo ""
echo -n "Enter terminal type [default: vt100]: "
read TERM

echo ""
echo "Possible speeds (in baud):"
echo "50 75 110 134 150 200 300 600 1200 1800 2400 4800 9600 19200 38400"
echo -n "Enter terminal speed [default: 2400]: "
read SPEED

echo ""
echo -n "Enter number of columns [default: 80]: "
read COLUMNS

echo ""
echo -n "Enter number of rows [default: 24]: "
read ROWS

echo ""
echo -n "Does your terminal have flow control [Y/N, default: Y]? "
read FLOW_CONTROL

echo ""
echo "Note: enabling tabs may cause unpredictable behavior for escape codes."
echo -n "Do you want tabs to be expanded to spaces [Y/N, default: N]? "
read TABS

echo ""
echo -n "Would you like to have your login times saved to a file [Y/N, default: N]? "
read LOG

echo ""
echo -n "Would you like the wuarchive archives mounted [Y/N, default: N]? " 
read WUARCHIVE

# To avoid saving the configuration to $HOME/.config each time, simply cd $HOME
cd $HOME

# Create .config and save the above terminal settings to it... "test -n" tests 
# to see if the variable exists, and 2>/dev/null redirects the error (if the 
# variable doesn't exist) to the null device.  This avoids output to the screen.

if (test -n $TERM 2>/dev/null) then
  echo "setenv TERM $TERM " >.config
else
  echo "setenv TERM vt100 " >.config
fi

if (test -n $SPEED 2>/dev/null) then
  echo -n "stty $SPEED " >>.config
else
  echo -n "stty 2400 " >>.config
fi

if (test -n $COLUMNS 2>/dev/null) then
  echo -n "columns $COLUMNS " >>.config
else
  echo -n "columns 80 " >>.config
fi

if (test -n $ROWS 2>/dev/null) then
  echo -n "rows $ROWS " >>.config
else
  echo -n "rows 24 " >>.config
fi

if (test $FLOW_CONTROL = "N" 2>/dev/null) then
  echo -n "-tandem " >>.config
else
  echo -n "tandem " >>.config
fi

if (test $TABS = "Y" 2>/dev/null) then
  echo "-tabs crt" >>.config
else
  echo "tabs crt" >>.config
fi

echo "Terminal settings saved; upon your next login, they will be loaded."

if (test $LOG = "Y" 2>/dev/null) then
  echo "echo -n \"Login:  \" >>.log" >>.config
  echo "date >>.log" >>.config

  echo "echo -n \"Logout: \" >>~/.log" >.logout
  echo "date >>~/.log" >>.logout

  echo "clear; echo \"Bye.\"" >>.logout
   
  echo "A logfile of all logins and logouts will be kept in ~/.log"
else
  echo "clear; echo \"Bye.\"" >.logout
fi

if (test -d wuarchive 2>/dev/null) then

# umount requires an absolute path
  umount $HOME/wuarchive/mirrors
  umount $HOME/wuarchive/mirrors2
  umount $HOME/wuarchive/mirrors3
  umount $HOME/wuarchive/mirrors4
  umount $HOME/wuarchive/systems
  umount $HOME/wuarchive/usenet
  rm -rf wuarchive
fi

if (test $WUARCHIVE = "Y" 2>/dev/null) then

# make the directories required to mount wuarchive
  mkdir wuarchive
  cd wuarchive
  mkdir mirrors mirrors2 mirrors3 mirrors4 systems usenet
  cd $HOME

# mount the wuarchive directories
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/mirrors $HOME/wuarchive/mirrors
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/mirrors2 $HOME/wuarchive/mirrors2
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/mirrors3 $HOME/wuarchive/mirrors3
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/mirrors4 $HOME/wuarchive/mirrors4   
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/systems $HOME/wuarchive/systems
  mount -o ro,bg,soft wuarchive.wustl.edu:/archive/usenet $HOME/wuarchive/usenet
  echo "Archives from wuarchive mounted in $HOME/wuarchive/"
fi



    







